// Winamp test visualization library v1.0
// Copyright (C) 1997-1998, Justin Frankel/Nullsoft
// Feel free to base any plugins on this "framework"...

#include <windows.h>
#include "..\winamp\wa_ipc.h"

#include "vis.h"

char szAppName[] = "K8055-Visual"; // Our window class, etc

typedef long(__stdcall *t_func)();

t_func t_OpenDevice;
t_func t_CloseDevice;
t_func t_ReadAllDigital;
t_func t_WriteAllDigital;
t_func t_OutputAllAnalog;
HINSTANCE hModule;


void config_read(struct winampVisModule *this_mod);		// reads the configuration
void config_write(struct winampVisModule *this_mod);	// writes the configuration
void config_getinifn(struct winampVisModule *this_mod, char *ini_file); // makes the .ini file filename

// returns a winampVisModule when requested. Used in hdr, below
winampVisModule *getModule(int which);

// "member" functions
void config(struct winampVisModule *this_mod); // configuration dialog
int init(struct winampVisModule *this_mod);	   // initialization for module
int render(struct winampVisModule *this_mod);  // rendering for module 1
void quit(struct winampVisModule *this_mod);   // deinitialization for module
int random_presets_flag = 0;

// our window procedure
LRESULT CALLBACK WndProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam);
HWND hMainWnd; // main window handle

// Double buffering data
HDC memDC;		// memory device context
HBITMAP	memBM,  // memory bitmap (for memDC)
		oldBM;  // old bitmap (from memDC)


// Module header, includes version, description, and address of the module retriever function
winampVisHeader hdr = { VIS_HDRVER, "K8055-Visualisation Plugin", getModule };

winampVisModule mod =
{
	"K8055-VU Meter",
	NULL,	// hwndParent
	NULL,	// hDllInstance
	0,		// sRate
	0,		// nCh
	100,		// latencyMS
	50,		// delayMS
	0,		// spectrumNch
	2,		// waveformNch
	{ 0, },	// spectrumData
	{ 0, },	// waveformData
	config,
	init,
	render, 
	quit
};


// this is the only exported symbol. returns our main header.
// if you are compiling C++, the extern "C" { is necessary, so we just #ifdef it
#ifdef __cplusplus
extern "C" {
#endif
__declspec( dllexport ) winampVisHeader *winampVisGetHeader()
{
	return &hdr;
}
#ifdef __cplusplus
}
#endif

// getmodule routine from the main header. Returns NULL if an invalid module was requested,
// otherwise returns either mod1, mod2 or mod3 depending on 'which'.
winampVisModule *getModule(int which)
{
	switch (which)
	{
		case 0: return &mod;
		default:return NULL;
	}
}

// configuration. Passed this_mod, as a "this" parameter. Allows you to make one configuration
// function that shares code for all your modules (you don't HAVE to use it though, you can make
// config1(), config2(), etc...)
void config(struct winampVisModule *this_mod)
{
	MessageBox(this_mod->hwndParent,"This module is Copyright (c) 1997-1998, Justin Frankel/Nullsoft\n"
									"-- This is just a demonstration module, it really isn't\n"
									"   supposed to be enjoyable --","Configuration",MB_OK);
}

embedWindowState myWindowState;

winampVisModule *g_mod = NULL;

// initialization. Registers our window class, creates our window, etc. Again, this one works for
// both modules, but you could make init1() and init2()...
// returns 0 on success, 1 on failure.
int init(struct winampVisModule *this_mod)
{
hModule = LoadLibrary("k8055d.dll");
t_OpenDevice = (t_func) GetProcAddress(hModule, "OpenDevice");    
t_ReadAllDigital = (t_func) GetProcAddress(hModule, "ReadAllDigital");
t_WriteAllDigital = (t_func) GetProcAddress(hModule, "WriteAllDigital");
t_CloseDevice = (t_func) GetProcAddress(hModule, "CloseDevice");
t_OutputAllAnalog = (t_func) GetProcAddress(hModule, "OutputAllAnalog");
t_OpenDevice(0);
}

int smoth=0;
float tmm=0;
int render(struct winampVisModule *this_mod)
{
int x,y=0;
int tempr,templ=0;
int lastr,lastl=0;

lastr=this_mod->waveformData[0][0];
lastl=this_mod->waveformData[1][0];

int total=0;
for (x = 1; x < 576; x ++)
{
total += abs(lastr - this_mod->waveformData[0][x]);
lastr = this_mod->waveformData[0][x];
}
total /= 288;
if (total > 127) total = 127;
tempr=total*2;

total=0;
for (x = 1; x < 576; x ++)
{
total += abs(lastl - this_mod->waveformData[1][x]);
lastl = this_mod->waveformData[1][x];
}
total /= 288;
if (total > 127) total = 127;
templ=total*2;


t_OutputAllAnalog((long)templ,(long)tempr);
tmm=(float) log((tempr+templ)/2)/2,4;
smoth=(int)(60*tmm);
int out=0;
if(smoth>0)out=1;
if(smoth>32)out=3;
if(smoth>64)out=7;
if(smoth>96)out=15;
if(smoth>128)out=31;
if(smoth>160)out=63;
if(smoth>192)out=127;
if(smoth>224)out=255;
t_WriteAllDigital(out);
return 0;
}

// cleanup (opposite of init()). Destroys the window, unregisters the window class
void quit(struct winampVisModule *this_mod)
{
t_CloseDevice(); 
}

void next_preset()
{
}

void previous_preset()
{
}

void load_random_preset()
{
}

void set_random(int r)
{
  random_presets_flag = r;
}

void go_fullscreen()
{
 
}

void go_windowed()
{
 
}

void open_configuration()
{
}

void open_popup_menu()
{
}


void config_getinifn(struct winampVisModule *this_mod, char *ini_file)
{
}


void config_read(struct winampVisModule *this_mod)
{
	
}

void config_write(struct winampVisModule *this_mod)
{
	
}
